"use strict";
/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var injectIntl_1 = require("./injectIntl");
var shallowEquals_ = require("shallow-equal/objects");
var shallowEquals = shallowEquals_;
var format_1 = require("../format");
var utils_1 = require("../utils");
var defaultFormatMessage = function (descriptor, values) {
    if (process.env.NODE_ENV !== 'production') {
        console.error('[React Intl] Could not find required `intl` object. <IntlProvider> needs to exist in the component ancestry. Using default message as fallback.');
    }
    return format_1.formatMessage(__assign({}, utils_1.DEFAULT_INTL_CONFIG, { locale: 'en' }), utils_1.createDefaultFormatters(), descriptor, values);
};
var BaseFormattedMessage = /** @class */ (function (_super) {
    __extends(BaseFormattedMessage, _super);
    function BaseFormattedMessage(props) {
        var _this = _super.call(this, props) || this;
        if (!props.defaultMessage) {
            utils_1.invariantIntlContext(props);
        }
        return _this;
    }
    BaseFormattedMessage.prototype.shouldComponentUpdate = function (nextProps) {
        var values = this.props.values;
        var nextValues = nextProps.values;
        if (!shallowEquals(nextValues, values)) {
            return true;
        }
        // Since `values` has already been checked, we know they're not
        // different, so the current `values` are carried over so the shallow
        // equals comparison on the other props isn't affected by the `values`.
        var nextPropsToCheck = __assign({}, nextProps, { values: values });
        return !shallowEquals(this.props, nextPropsToCheck);
    };
    BaseFormattedMessage.prototype.render = function () {
        var _a = this.props.intl || {}, _b = _a.formatMessage, formatMessage = _b === void 0 ? defaultFormatMessage : _b, _c = _a.textComponent, Text = _c === void 0 ? React.Fragment : _c;
        var _d = this.props, id = _d.id, description = _d.description, defaultMessage = _d.defaultMessage, values = _d.values, _e = _d.tagName, Component = _e === void 0 ? Text : _e, children = _d.children;
        var tokenDelimiter = '';
        var tokenizedValues = {};
        var elements = {};
        var hasValues = values && Object.keys(values).length > 0;
        if (hasValues) {
            // Creates a token with a random UID that should not be guessable or
            // conflict with other parts of the `message` string.
            var uid_1 = Math.floor(Math.random() * 0x10000000000).toString(16);
            var generateToken_1 = (function () {
                var counter = 0;
                return function () { return "ELEMENT-" + uid_1 + "-" + (counter += 1); };
            })();
            // Splitting with a delimiter to support IE8. When using a regex
            // with a capture group IE8 does not include the capture group in
            // the resulting array.
            tokenDelimiter = "@__" + uid_1 + "__@";
            // Iterates over the `props` to keep track of any React Element
            // values so they can be represented by the `token` as a placeholder
            // when the `message` is formatted. This allows the formatted
            // message to then be broken-up into parts with references to the
            // React Elements inserted back in.
            Object.keys(values).forEach(function (name) {
                var value = values[name];
                if (React.isValidElement(value)) {
                    var token = generateToken_1();
                    tokenizedValues[name] = tokenDelimiter + token + tokenDelimiter;
                    elements[token] = value;
                }
                else {
                    tokenizedValues[name] = value;
                }
            });
        }
        var descriptor = { id: id, description: description, defaultMessage: defaultMessage };
        var formattedMessage = formatMessage(descriptor, tokenizedValues || values);
        var nodes;
        var hasElements = elements && Object.keys(elements).length > 0;
        if (hasElements) {
            // Split the message into parts so the React Element values captured
            // above can be inserted back into the rendered message. This
            // approach allows messages to render with React Elements while
            // keeping React's virtual diffing working properly.
            nodes = formattedMessage
                .split(tokenDelimiter)
                .filter(function (part) { return !!part; })
                .map(function (part) { return elements[part] || part; });
        }
        else {
            nodes = [formattedMessage];
        }
        if (typeof children === 'function') {
            return children.apply(void 0, nodes);
        }
        // Needs to use `createElement()` instead of JSX, otherwise React will
        // warn about a missing `key` prop with rich-text message formatting.
        return React.createElement.apply(React, [Component, null].concat(nodes));
    };
    BaseFormattedMessage.defaultProps = {
        values: {},
    };
    return BaseFormattedMessage;
}(React.Component));
exports.BaseFormattedMessage = BaseFormattedMessage;
exports.default = injectIntl_1.default(BaseFormattedMessage, { enforceContext: false });
