"use strict";
/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var injectIntl_1 = require("./injectIntl");
var invariant_ = require("invariant");
// Since rollup cannot deal with namespace being a function,
// this is to interop with TypeScript since `invariant`
// does not export a default
// https://github.com/rollup/rollup/issues/1267
var invariant = invariant_;
var utils_1 = require("../utils");
var format_1 = require("../format");
var intl_locales_supported_1 = require("intl-locales-supported");
var shallowEquals_ = require("shallow-equal/objects");
var shallowEquals = shallowEquals_;
var intlConfigPropNames = [
    'locale',
    'timeZone',
    'formats',
    'messages',
    'textComponent',
    'defaultLocale',
    'defaultFormats',
    'onError',
];
function getConfig(filteredProps) {
    var config = __assign({}, utils_1.DEFAULT_INTL_CONFIG, filteredProps);
    // Apply default props. This must be applied last after the props have
    // been resolved and inherited from any <IntlProvider> in the ancestry.
    // This matches how React resolves `defaultProps`.
    for (var propName in utils_1.DEFAULT_INTL_CONFIG) {
        if (config[propName] === undefined) {
            config[propName] =
                utils_1.DEFAULT_INTL_CONFIG[propName];
        }
    }
    if (!config.locale || !intl_locales_supported_1.default(config.locale)) {
        var locale = config.locale, defaultLocale = config.defaultLocale, defaultFormats = config.defaultFormats, onError = config.onError;
        if (typeof onError === 'function')
            onError(utils_1.createError("Missing locale data for locale: \"" + locale + "\". " +
                ("Using default locale: \"" + defaultLocale + "\" as fallback.")));
        // Since there's no registered locale data for `locale`, this will
        // fallback to the `defaultLocale` to make sure things can render.
        // The `messages` are overridden to the `defaultProps` empty object
        // to maintain referential equality across re-renders. It's assumed
        // each <FormattedMessage> contains a `defaultMessage` prop.
        config = __assign({}, config, { locale: defaultLocale, formats: defaultFormats, messages: utils_1.DEFAULT_INTL_CONFIG.messages });
    }
    return config;
}
function getBoundFormatFns(config, state) {
    var formatterState = __assign({}, state.context.formatters);
    return {
        formatNumber: format_1.formatters.formatNumber.bind(undefined, config, formatterState),
        formatRelativeTime: format_1.formatters.formatRelativeTime.bind(undefined, config, formatterState),
        formatDate: format_1.formatters.formatDate.bind(undefined, config, formatterState),
        formatTime: format_1.formatters.formatTime.bind(undefined, config, formatterState),
        formatPlural: format_1.formatters.formatPlural.bind(undefined, config, formatterState),
        formatMessage: format_1.formatters.formatMessage.bind(undefined, config, formatterState),
        formatHTMLMessage: format_1.formatters.formatHTMLMessage.bind(undefined, config, formatterState),
    };
}
var IntlProvider = /** @class */ (function (_super) {
    __extends(IntlProvider, _super);
    function IntlProvider(props) {
        var _this = _super.call(this, props) || this;
        invariant(typeof Intl !== 'undefined', '[React Intl] The `Intl` APIs must be available in the runtime, ' +
            'and do not appear to be built-in. An `Intl` polyfill should be loaded.\n' +
            'See: http://formatjs.io/guides/runtime-environments/');
        var intlContext = props.intl;
        // Creating `Intl*` formatters is expensive. If there's a parent
        // `<IntlProvider>`, then its formatters will be used. Otherwise, this
        // memoize the `Intl*` constructors and cache them for the lifecycle of
        // this IntlProvider instance.
        var _a = (intlContext || {}).formatters, formatters = _a === void 0 ? utils_1.createDefaultFormatters() : _a;
        _this.state = {
            context: __assign({}, intlContext, { formatters: formatters }),
        };
        return _this;
    }
    IntlProvider.getDerivedStateFromProps = function (nextProps, prevState) {
        var intlContext = nextProps.intl;
        // Build a whitelisted config object from `props`, defaults, and
        // `props.intl`, if an <IntlProvider> exists in the ancestry.
        var filteredProps = utils_1.filterProps(nextProps, intlConfigPropNames, intlContext || {});
        if (!shallowEquals(filteredProps, prevState.filteredProps)) {
            var config = getConfig(filteredProps);
            var boundFormatFns = getBoundFormatFns(config, prevState);
            return {
                filteredProps: filteredProps,
                context: __assign({}, prevState.context, config, boundFormatFns),
            };
        }
        return null;
    };
    IntlProvider.prototype.render = function () {
        return (React.createElement(injectIntl_1.Provider, { value: this.state.context }, this.props.children));
    };
    return IntlProvider;
}(React.PureComponent));
exports.default = injectIntl_1.default(IntlProvider, { enforceContext: false }); // to be able to inherit values from parent providers
